#! /bin/sh

# Attempt to find a solution with small coefficients and rounds
# usage:
# $0 [nnn]
# attempt to find solutions with numbers <= nnn (11 by default)

BOUND=${1:-11}

function doit() {

  # generate smt encoding
  ./simplex-dpll > test.smt || { echo "generator failed"; return; }

  # check that we have 8 rounds (the generator may produce more)
  grep -q r10 test.smt && { echo "generator produced more than 8 rounds"; return; }

  # invoke SAT solver
  yices-smt2 test.smt > test.sol || { echo "SMT solver failed"; return; }

  # check whether a SAT result was produced
  grep -q ^unsat test.sol && { echo "UNSAT"; return; }
  grep -q ^sat test.sol || { echo "SMT solver failed?"; return; }

  # find maximum number in resulting assignments
  MAX=$(< test.sol sed 's= \([0-9]*\)=\n\1\n=g' | grep '^[0-9]*$' | sort -n | tail -n 1)
  echo SAT \($MAX\)

  # check success
  [[ $MAX -le $BOUND ]] && { echo SUCCESS, see test.smt and test.sol; exit; }
}

$P ghc --make -DROUNDS=2 -DCYCLES=2 simplex-dpll

for i in $(seq 10000); do
  doit
done
